#include "SplitFlap.h"
#include <AccelStepper.h> //https://github.com/waspinator/AccelStepper, reference: http://www.airspayce.com/mikem/arduino/AccelStepper/classAccelStepper.html
#include <Arduino.h>
#include <Wire.h>

const byte CALIBRATE = 1;
const byte GO_TO_LETTER = 2;
const byte SPIN = 3;
const byte TEST = 4;

const int FollowerAddress = 9;
const int hallPin = 2;
volatile bool startFound = false;

char flap[] = {'<', '.', '?', '!', '`', '-', '(', ')', '1', '2', '3', '4', '5', '6', '7', '8', '9', '0', '^', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', '>', '#', '@', '$', '%'};

const int stepsPerIndex = 8; 
const int stepsPerRevolution = 200;
const int speed = 100;

int currFlap = 0;
bool completedCurrentState = false;
bool ledTest = LOW;
const int offset = 20;

AccelStepper module0motor = AccelStepper(AccelStepper::DRIVER, 4, 3); //step pin, direction pin
SplitFlap module0 = SplitFlap( //a split flap needs a pointer to an accelStepper and pins and info
    &module0motor, //https://www.airspayce.com/mikem/arduino/AccelStepper/classAccelStepper.html#a3bc75bd6571b98a6177838ca81ac39ab
    2, stepsPerRevolution, speed, 100, 50, offset); //zero sensor pin, stepsPerRev, speed, accel, numberOfFlaps, zeroPositionOffset
SF_MAKE_ISR_MACRO(module0); //create interrupt service routine

enum FollowerState {
  Waiting,
  Calibrating,
  SearchingForLetter,
  Spinning,
};
FollowerState state;

void setup()
{
    module0.setUpInterrupts(module0_SF_ISR);
    module0.begin();
    Wire.begin(FollowerAddress);
    Wire.onRequest(requestEvent); 
    Wire.onReceive(receiveEvent); 

    pinMode(LED_BUILTIN, OUTPUT);

    Serial.begin(9600);
    Serial.println("follower");

    currFlap = 0;
    completedCurrentState = false;
    state = Waiting;
}

void loop() {
  switch (state) {
    case (Calibrating):
      if(completedCurrentState == false && !module0.motor->isRunning()) {
        completedCurrentState = true;
        state = Waiting;
      }
      module0.run();
      break;
    case (SearchingForLetter):
      if(completedCurrentState == false && !module0.motor->isRunning()) {
        completedCurrentState = true;
        state = Waiting;
      }
      module0.run();
      break;
    case (Spinning):
      module0.runSpeed();
      break;
    default:
      module0.run();
      break;
  }
}

void requestEvent() {
    if(completedCurrentState){
        Wire.write(1);
    } else {
        Wire.write(0);
    }
}

void receiveEvent(int numBytes){
    Serial.println("received event");
    completedCurrentState = false;
    int command;
    char value;
    while (1 < Wire.available()){
      command = Wire.read();
    }
    value = Wire.read();
    Serial.println(value);
    switch(command) {
      case CALIBRATE:
          calibrate();
          break;
      case GO_TO_LETTER:
          goToLetter(value);
          break;
      case SPIN:
          toggleSpin();
      case TEST:
          test();
          break;
      default:
        break;
    }
}

void calibrate() { 
  Serial.println("calibrating");
  module0.display(0);
  state = Calibrating;
}

void goToLetter(char target) {
    int targetFlap = 0; // defines index of target flap
    for (int i = 0; i < 50; i++) {
        if (flap[i] == target) {
          targetFlap = i; 
        }
    }
    module0.display(targetFlap); 
    currFlap = targetFlap;
    state = SearchingForLetter;
}

void toggleSpin() {
  if(state == Spinning) {
    state = Waiting;
  }
  else {
    module0.setSpeed(speed);
    state = Spinning;
  }
}

void test() {
  Serial.println("test");
  digitalWrite(LED_BUILTIN, ledTest);
  ledTest = !ledTest;
  completedCurrentState = true;
  Serial.print("completed test");
}
